#include "ZTM_DEF.h"


void WRITE_COLLISION(ofstream * file);
void WRITE_TEXTURES(ofstream * binFile);


unsigned short convert_to_4bpp(unsigned short a, unsigned short b){    return (((a&0xff)<<4) | (b));}

/**The model data is just the header portion of the model - total of 36 bytes**/
void WRITE_MDATA(ofstream * file, _MOD_DATA * model)
{
    unsigned short bUint16;
    unsigned int bUint32;
    unsigned int bSint32;


    bUint16=swap_endian_ushort(model->TOTAL_MESH);
        file->write((char*)&bUint16, sizeof(unsigned short));

    bUint16=swap_endian_sshort(model->TOT_TEXT);
        file->write((char*)&bUint16, sizeof(unsigned short));

    bUint32=swap_endian_uint(model->TEXT_SIZE);
        file->write((char*)&bUint32, sizeof(unsigned int));

    bUint32=swap_endian_uint(model->PDATA_SIZE);
        file->write((char*)&bUint32, sizeof(unsigned int));


    bSint32=swap_endian_sint(model->Origin.x);
        file->write((char*)&bSint32, sizeof(int));
    bSint32=swap_endian_sint(model->Origin.y);
        file->write((char*)&bSint32, sizeof(int));
    bSint32=swap_endian_sint(model->Origin.z);
        file->write((char*)&bSint32, sizeof(int));

    bSint32=swap_endian_sint(model->Length.x);
        file->write((char*)&bSint32, sizeof(int));
    bSint32=swap_endian_sint(model->Length.y);
        file->write((char*)&bSint32, sizeof(int));
    bSint32=swap_endian_sint(model->Length.z);
        file->write((char*)&bSint32, sizeof(int));

    //Animation : Updated 2018/05/31
    bUint16=swap_endian_sshort(model->nbFrames);
        file->write((char*)&bUint16, sizeof(unsigned short));
    bUint16=swap_endian_sshort(model->framerate);
        file->write((char*)&bUint16, sizeof(unsigned short));

    bSint32=swap_endian_sint(0x00000000);
        file->write((char*)&bSint32, sizeof(int));
}


/*****
This writes all the PDATA in a sequential order
*****/
void WRITE_PDATA(ofstream * file, _MOD_DATA * model)
{
    unsigned short bUint16;
    //short bSint16;
    unsigned int bUint32;
    int bSint32;

    for (unsigned int i=0; i<model->TOTAL_MESH; i++)
    {
        //PDATA, including buffers for the pointers
        bUint32 = 0;
            file->write((char*)&bUint32, sizeof(unsigned int));
        bUint32 = swap_endian_uint(model->pol[i].nbPoint);
            file->write((char*)&bUint32, sizeof(unsigned int));
        bUint32 = 0;
            file->write((char*)&bUint32, sizeof(unsigned int));
        bUint32 = swap_endian_uint(model->pol[i].nbPolygon);
            file->write((char*)&bUint32, sizeof(unsigned int));
        bUint32 = 0;
            file->write((char*)&bUint32, sizeof(unsigned int));
        bUint32 = 0; //VECTOR TABLE
            file->write((char*)&bUint32, sizeof(unsigned int));

        //POINT, 12 bytes each
        for (unsigned int ii=0; ii<model->pol[i].nbPoint; ii++)
        {
            for (unsigned int j=0; j<3; j++)
            {
                bSint32 = swap_endian_sint(model->pol[i].pntbl[ii][j]);
                file->write((char*)&bSint32, sizeof(int));
            }
        }
        //POLYGON, 12 bytes for normals and 8 bytes for vertices
        for (unsigned int ii=0; ii<model->pol[i].nbPolygon; ii++)
        {
            //Normals
            for (unsigned int j=0; j<3; j++)
            {
                bSint32 = swap_endian_sint(model->pol[i].pltbl[ii].norm[j]);
                file->write((char*)&bSint32, sizeof(int));
            }
            //Vertices
            for (unsigned int j=0; j<4; j++)
            {
                bUint16 = swap_endian_ushort(model->pol[i].pltbl[ii].Vertices[j]);
                file->write((char*)&bUint16, sizeof(unsigned short));
            }
        }
        //ATTR, 12 bytes each
        for (unsigned int ii=0; ii<model->pol[i].nbPolygon; ii++)
        {
            file->write((char*)&TextureAttributeArray[model->pol[i].attbl[ii].texno].flag, sizeof(unsigned char));
            file->write((char*)&TextureAttributeArray[model->pol[i].attbl[ii].texno].sorting, sizeof(unsigned char));
            bUint16=swap_endian_ushort(TextureAttributeArray[model->pol[i].attbl[ii].texno].texno);
                file->write((char*)&bUint16, sizeof(unsigned short));
            bUint16=swap_endian_ushort(TextureAttributeArray[model->pol[i].attbl[ii].texno].atrb);
                file->write((char*)&bUint16, sizeof(unsigned short));
            bUint16=swap_endian_ushort(TextureAttributeArray[model->pol[i].attbl[ii].texno].colno);
                file->write((char*)&bUint16, sizeof(unsigned short));
            bUint16=swap_endian_ushort(TextureAttributeArray[model->pol[i].attbl[ii].texno].gstb);
                file->write((char*)&bUint16, sizeof(unsigned short));
            bUint16=swap_endian_ushort(TextureAttributeArray[model->pol[i].attbl[ii].texno].dir);
                file->write((char*)&bUint16, sizeof(unsigned short));
        }
        //VECTOR NORMALS

        for (unsigned int ii=0; ii<model->pol[i].nbPoint; ii++) //VECTOR : 12 bytes each
        {
            for (unsigned int j=0; j<3; j++)
            {
                bSint32 = swap_endian_sint(model->pol[i].vntbl[ii][j]);
                file->write((char*)&bSint32, sizeof(int));
            }
        }
    }
}



void WRITE_TEXTURES(ofstream * binFile, _MOD_DATA * model)
{
    cout << "Writing the textures to binary file...\n\n";
    unsigned short buf16;

    for (short i=0; i<model->TOT_TEXT; i++)
    {
        unsigned short w = swap_endian_ushort(pimg[i].width);
        unsigned short h = swap_endian_ushort(pimg[i].height);
        binFile->write((char*)&w, sizeof(unsigned short));
        binFile->write((char*)&h, sizeof(unsigned short));
        buf16 = swap_endian_ushort(COL_16);
        binFile->write((char*)(&buf16), sizeof(unsigned short));
        buf16 = swap_endian_ushort(16);
        binFile->write((char*)(&buf16), sizeof(unsigned short));
        binFile->write((char*)&model->TEXT_SIZE, sizeof(int));

        for (short ii=0; ii< (pimg[i].width * pimg[i].height);)
        {
            unsigned char buf = 0;
            buf =  (unsigned char) convert_to_4bpp(pimg[i].palette_id[ii], pimg[i].palette_id[ii+1]);
            binFile->write((char*)&buf, sizeof(unsigned char));
            ii+=2;
        }

    }
    for (short i=0; i<model->TOT_TEXT; i++)
    {
        for (unsigned int ii=0; ii<16; ii++)
        {
           buf16 = swap_endian_ushort(pimg[i].palette[ii]);
           binFile->write((char*)(&buf16), sizeof(unsigned short));
        }
    }
}

/********************************
* WIP : Writes data to custom binary format
*********************************/
void write_map_to_bin(ofstream * file, _MOD_DATA * model)
{
    /*
    cout << "\nWRITE MODEL TO BINARY .ZTP FILE - Beta\n";
    string fileOutName = filename + ".ZTP";  //Z-Treme character
    ofstream binFile(fileOutName.c_str(), ios::out | ios::binary);
    if (!binFile.is_open()) {cout << "\nERROR WHILE WRITING BINARY FILE!\n"; return;}
    */

    WRITE_MDATA(file, model);
    WRITE_TEXTURES(file, model);
    WRITE_PDATA(file, model);
   // WRITE_ANIMATION(file, model);
    //binFile.close();
}




