#include "ZTM_DEF.h"


#define test_convert(x) (x)//(x/65536)




bool write_MOD_HEADER(string filename, _MOD_DATA * model)
{
    ofstream file(filename, ios::out);
    if (!file.is_open()) {cout << "ERROR...\n"; return false;}

    file << "#ifndef LUTaddr\n";
    file << "#define     LUTaddr             (0x25C7A960)\n";
    file << "#define     returnLUTaddr(n)    (LUTaddr + (0x20 * n))\n";
    file << "#define     LUTidx(n)           (Uint16)(62764 + (4 * n))\n";
    file << "#define     CRAM_Base           (0x25f00200)\n";
    file << "#define     returnCRAMaddr(n)   (CRAM_Base + (0x20 * n))\n";
    file << "#define     LUTcramIdx(n)       ((256 + (16 * n)))\n";
    file << "#endif\n\n";

    for (unsigned int mesh=0; mesh<model->TOTAL_MESH; mesh++)
    {
        if (model->pol[mesh].nbPolygon > 0)
        {
            file << "//MESH NO." << mesh << " : " << PdataName[mesh] << "\n";

            file << "POINT point_" << mesh << "[] = {\n";
            for (unsigned int i=0; i<model->pol[mesh].nbPoint; i++)
            {
                file << "       POStoFIXED(" << toFLOAT(test_convert(model->pol[mesh].pntbl[i][X])) << ", "
                << toFLOAT(test_convert(model->pol[mesh].pntbl[i][Y])) << ", "
                << toFLOAT(test_convert(model->pol[mesh].pntbl[i][Z])) << "),\n";
            }
            file << "};\n\n";

            file << "POLYGON polygon_" << mesh << "[] = {\n";
            for (unsigned int i=0; i<model->pol[mesh].nbPolygon; i++)
            {
                file << "       NORMAL(";
                for (unsigned int j=0; j<3; j++)
                {file << toFLOAT(model->pol[mesh].pltbl[i].norm[j]); if (j<2) file << ", ";}
                file << "), VERTICES(";
                for (unsigned int j=0; j<4; j++)
                {file << model->pol[mesh].pltbl[i].Vertices[j]; if (j<3) file << ", ";}
                file << "),\n";
            }
            file << "};\n\n";

            file << "ATTR attribute_" << mesh << "[] = {\n";
            for (unsigned int i=0; i<model->pol[mesh].nbPolygon; i++)
            {
                file << "       ATTRIBUTE(";
                file << "Single_Plane, SORT_CEN, ";
                file << model->pol[mesh].attbl[i].texno << ", LUTidx(" << model->pol[mesh].attbl[i].texno << "),";
                file << " No_Gouraud,Window_In|MESHoff|HSSon|ECdis | SPdis |CL16Look,sprNoflip,No_Option|UseNearClip),\n";
            }
            file << "};\n\n";

            file << "VECTOR normal_" << mesh << "[] = {\n";
            for (unsigned int i=0; i<model->pol[mesh].nbPoint; i++)
            {
                file << "       POStoFIXED(" << toFLOAT(test_convert(model->pol[mesh].vntbl[i][X]))
                << ", " << toFLOAT(test_convert(model->pol[mesh].vntbl[i][Y])) << ", "
                << toFLOAT(test_convert(model->pol[mesh].vntbl[i][Z])) << "),\n";
            }
            file << "};\n\n";

            file << "XPDATA xpdata_" << mesh << "[] = {\n   {\n";
            file << "       point_" << mesh << ", sizeof(point_" << mesh << ")/sizeof(POINT),\n";
            file << "       polygon_" << mesh << ", sizeof(polygon_" << mesh << ")/sizeof(POLYGON),\n";
            file << "       attribute_" << mesh << ",\n       normal_" << mesh << "\n  }\n";
            file << "};\n\n";


        }
    }

    unsigned int cntPnt = 0;
    unsigned int cntPlt = 0;

    for (unsigned int mesh=0; mesh<model->TOTAL_MESH; mesh++)
    {
        cntPnt += model->pol[mesh].nbPoint;
        cntPlt += model->pol[mesh].nbPolygon;
    }
    file << "//TOTAL POINTS : " << cntPnt << "\n";
    file << "//TOTAL POLYGONS : " << cntPlt << "\n";


    file << "#ifndef _ANORM_h\n#define _ANORM_h\n";
    file << "\nVECTOR ANORMS[]={\n";
    for (int i=0; i<model->pol[0].nbPoint; i++)
    {
        float buf[3];
        buf[X]=toFLOAT(model->pol[0].pntbl[i][X]);
        buf[Y]=toFLOAT(model->pol[0].pntbl[i][Y]);
        buf[Z]=toFLOAT(model->pol[0].pntbl[i][Z]);
        float dist=sqrtf((buf[X]*buf[X])+(buf[Y]*buf[Y])+(buf[Z]*buf[Z]));
        if (dist != 0)
        {
            buf[X]= (buf[X]/dist);
            buf[Y]= (buf[Y]/dist);
            buf[Z]= (buf[Z]/dist);
        }
        file << "POStoFIXED(" << buf[X] << ", " << buf[Y] << ", " << buf[Z] << "),\n";
    }
    file << "};";

    file.close();
    return true;
}


